@@ -58,7 +58,7 @@ def is_sentencepiece_installed():
 
 
 def get_cflags_and_libs(root):
-  cflags = ['-std=c++11', '-I' + os.path.join(root, 'include')]
+  cflags = ['-std=c++17', '-I' + os.path.join(root, 'include')]
   libs = []
   if os.path.exists(os.path.join(root, 'lib/pkgconfig/sentencepiece.pc')):
     libs = [
@@ -78,8 +78,6 @@ class build_ext(_build_ext):
 
   def build_extension(self, ext):
     cflags, libs = get_cflags_and_libs('../build/root')
-    if len(libs) == 0:
-      cflags, libs = get_cflags_and_libs('./bundled/root')
 
     if len(libs) == 0:
       if is_sentencepiece_installed():
@@ -87,12 +85,15 @@ class build_ext(_build_ext):
         libs = run_pkg_config('libs')
       else:
         subprocess.check_call(['./build_bundled.sh', __version__])
-        cflags, libs = get_cflags_and_libs('./bundled/root')
+        cflags, libs = get_cflags_and_libs('./build/root')
 
     # Fix compile on some versions of Mac OSX
     # See: https://github.com/neulab/xnmt/issues/199
     if sys.platform == 'darwin':
       cflags.append('-mmacosx-version-min=10.9')
+    else:
+      cflags.append('-Wl,-strip-all')
+      libs.append('-Wl,-strip-all')
     print('## cflags={}'.format(' '.join(cflags)))
     print('## libs={}'.format(' '.join(libs)))
     ext.extra_compile_args = cflags
@@ -101,33 +102,66 @@ class build_ext(_build_ext):
 
 
 if os.name == 'nt':
-  # Must pre-install sentencepice into bundled directory.
+  # Must pre-install sentencepice into build directory.
   arch = 'win32'
   if sys.maxsize > 2**32:
     arch = 'amd64'
   if os.path.exists('..\\build\\root_{}\\lib'.format(arch)):
-    cflags = ['/MT', '/I..\\build\\root_{}\\include'.format(arch)]
+    cflags = ['/std:c++17', '/I..\\build\\root_{}\\include'.format(arch)]
     libs = [
         '..\\build\\root_{}\\lib\\sentencepiece.lib'.format(arch),
-        '..\\build\\root_{}\\lib\\sentencepiece_train.lib'.format(arch)
+        '..\\build\\root_{}\\lib\\sentencepiece_train.lib'.format(arch),
     ]
-  else:
-    cflags = ['/MT', '/I..\\build\\root\\include']
+  elif os.path.exists('..\\build\\root\\lib'):
+    cflags = ['/std:c++17', '/I..\\build\\root\\include']
     libs = [
         '..\\build\\root\\lib\\sentencepiece.lib',
-        '..\\build\\root\\lib\\sentencepiece_train.lib'
+        '..\\build\\root\\lib\\sentencepiece_train.lib',
+    ]
+  else:
+    # build library locally with cmake and vc++.
+    cmake_arch = 'Win32'
+    if arch == 'amd64':
+      cmake_arch = 'x64'
+    subprocess.check_call([
+        'cmake',
+        'sentencepiece',
+        '-A',
+        cmake_arch,
+        '-B',
+        'build',
+        '-DSPM_ENABLE_SHARED=OFF',
+        '-DCMAKE_INSTALL_PREFIX=build\\root',
+    ])
+    subprocess.check_call([
+        'cmake',
+        '--build',
+        'build',
+        '--config',
+        'Release',
+        '--target',
+        'install',
+        '--parallel',
+        '8',
+    ])
+    cflags = ['/std:c++17', '/I.\\build\\root\\include']
+    libs = [
+        '.\\build\\root\\lib\\sentencepiece.lib',
+        '.\\build\\root\\lib\\sentencepiece_train.lib',
     ]
 
   SENTENCEPIECE_EXT = Extension(
       'sentencepiece._sentencepiece',
       sources=['src/sentencepiece/sentencepiece_wrap.cxx'],
       extra_compile_args=cflags,
-      extra_link_args=libs)
+      extra_link_args=libs,
+  )
   cmdclass = {}
 else:
   SENTENCEPIECE_EXT = Extension(
       'sentencepiece._sentencepiece',
-      sources=['src/sentencepiece/sentencepiece_wrap.cxx'])
+      sources=['src/sentencepiece/sentencepiece_wrap.cxx'],
+  )
   cmdclass = {'build_ext': build_ext}
 
 setup(
@@ -143,19 +177,23 @@ setup(
     license='Apache',
     platforms='Unix',
     py_modules=[
-        'sentencepiece/__init__', 'sentencepiece/_version',
+        'sentencepiece/__init__',
+        'sentencepiece/_version',
         'sentencepiece/sentencepiece_model_pb2',
-        'sentencepiece/sentencepiece_pb2'
+        'sentencepiece/sentencepiece_pb2',
     ],
     ext_modules=[SENTENCEPIECE_EXT],
     cmdclass=cmdclass,
     classifiers=[
-        'Development Status :: 5 - Production/Stable', 'Environment :: Console',
+        'Development Status :: 5 - Production/Stable',
+        'Environment :: Console',
         'Intended Audience :: Developers',
         'Intended Audience :: Science/Research',
         'License :: OSI Approved :: Apache Software License',
-        'Operating System :: Unix', 'Programming Language :: Python',
+        'Operating System :: Unix',
+        'Programming Language :: Python',
         'Topic :: Text Processing :: Linguistic',
-        'Topic :: Software Development :: Libraries :: Python Modules'
+        'Topic :: Software Development :: Libraries :: Python Modules',
     ],
-    test_suite='sentencepiece_test.suite')
+    test_suite='sentencepiece_test.suite',
+)
