@@ -119,8 +119,10 @@ TEST(UnigramTrainerTest, BasicTest) {
   // Check seed pieces.
   EXPECT_EQ(27, res.seed_pieces_and_probs.size());
 
+  LOG(INFO) << "[" << res.sentence_pieces << "]";
+
   // Check final pieces.
-  EXPECT_EQ("i a n y m l e apple ve O P r t g an v ▁ A b le ▁an p d h",
+  EXPECT_EQ("i a n y m l e apple ve O P r g t an v ▁ b A le ▁an p d h",
             res.sentence_pieces);
 }
 
@@ -191,12 +193,13 @@ TEST(UnigramTrainerTest, EndToEndTest) {
                   .ok());
   // TODO(taku): Temporally disable this test on Windows.
 #ifndef OS_WIN
-  EXPECT_EQ(WS
-            " 吾輩 《 わが はい 》 は 猫 である 。 名前 はまだ 無い 。 "
-            "どこ で 生 れた か とん と 見当 《 けん とう 》 が つか ぬ 。 "
-            "何でも 薄 暗 い じめ じめ した 所で ニャーニャー "
-            "泣 い ていた 事 だけは 記憶 している 。",
-            absl::StrJoin(tok, " "));
+  LOG(INFO) << "[" << absl::StrJoin(tok, " ") << std::endl;
+  EXPECT_EQ(
+      WS
+      " 吾輩 《 わが はい 》 は猫である 。 名前はまだ 無 い 。 どこ で 生 れた "
+      "か とん と 見当 《 けん とう 》 が つか ぬ 。 何でも 薄 暗 い じめ じめ "
+      "した 所で ニャーニャー 泣 い ていた 事 だけは 記憶 している 。",
+      absl::StrJoin(tok, " "));
 #endif
 }
 
